@@ -1,6 +1,6 @@
-// application.rs
-//
-// Copyright 2020 Christopher Davis <christopherdavis@gnome.org>
+// Copyright (c) 2020-2023 Christopher Davis
+// Copyright (c) 2022-2023 Sophie Herold
+// Copyright (c) 2022 Sabri Ünal
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use crate::deps::*;
-use crate::i18n::*;
+use crate::util::gettext::*;
 
 use adw::prelude::*;
 use adw::subclass::prelude::*;
@@ -72,10 +72,8 @@ mod imp {
         fn open(&self, files: &[gio::File], _hint: &str) {
             let application = self.obj();
             for file in files {
-                let Some(path) = file.path() else { log::error!("File has no path {}", file.uri()); return; };
-
                 let win = LpWindow::new(&*application);
-                win.set_image_from_path(&path);
+                win.image_view().set_image_from_file(file.clone());
             }
         }
     }
@@ -149,8 +147,14 @@ impl LpApplication {
         self.set_accels_for_action("win.first", &["Home"]);
         self.set_accels_for_action("win.last", &["End"]);
 
-        self.set_accels_for_action("win.zoom-to(1.0)", &["1", "KP_1", "<Ctrl>1", "<Ctrl>KP_1"]);
-        self.set_accels_for_action("win.zoom-to(2.0)", &["2", "KP_2", "<Ctrl>2", "<Ctrl>KP_2"]);
+        self.set_accels_for_action(
+            "win.zoom-to-exact(1.0)",
+            &["1", "KP_1", "<Ctrl>1", "<Ctrl>KP_1"],
+        );
+        self.set_accels_for_action(
+            "win.zoom-to-exact(2.0)",
+            &["2", "KP_2", "<Ctrl>2", "<Ctrl>KP_2"],
+        );
         self.set_accels_for_action("win.zoom-best-fit", &["0", "KP_0", "<Ctrl>0", "<Ctrl>KP_0"]);
         self.set_accels_for_action(
             "win.zoom-in",
@@ -184,10 +188,10 @@ impl LpApplication {
         // an object and set all relevant properties very
         // easily in a way that's idiomatic to Rust.
         let about = adw::AboutWindow::builder()
-            .application_name(i18n("Loupe"))
+            .application_name(gettext("Loupe"))
             .application_icon(config::APP_ID)
             .version(config::VERSION)
-            .developer_name(i18n("The Loupe Team"))
+            .developer_name(gettext("The Loupe Team"))
             .website("https://gitlab.gnome.org/Incubator/loupe")
             .issue_url("https://gitlab.gnome.org/Incubator/loupe/-/issues/new")
             .developers([
@@ -196,8 +200,8 @@ impl LpApplication {
             ])
             .designers(["Allan Day", "Jakub Steiner", "Tobias Bernard"])
             // Translators: Replace "translator-credits" with your names, one name per line
-            .translator_credits(i18n("translator-credits"))
-            .copyright(i18n("Copyright © 2020–2023 Christopher Davis et al."))
+            .translator_credits(gettext("translator-credits"))
+            .copyright(gettext("Copyright © 2020–2023 Christopher Davis et al."))
             .license_type(gtk::License::Gpl30)
             .build();
 
