@@ -1,17 +1,34 @@
+// Copyright (c) 2023 Sophie Herold
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 ///! Decode using image-rs
 use super::*;
 use crate::decoder::tiling::{self, FrameBufferExt};
 use crate::deps::*;
-use crate::i18n::*;
+use crate::util;
+use crate::util::gettext::*;
+use crate::util::{BufReadSeek, ToBufRead};
 
-use anyhow::Context;
+use anyhow::{bail, Context};
 use arc_swap::ArcSwap;
 use gtk::prelude::*;
-use image_rs::AnimationDecoder;
+use image_rs::{AnimationDecoder, ImageDecoder};
+use rgb::AsPixels;
 
-use std::fs::File;
-use std::io::BufReader;
-use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
 #[derive(Debug)]
@@ -23,8 +40,12 @@ pub struct ImageRsOther {
 impl Drop for ImageRsOther {
     fn drop(&mut self) {
         if let Some(handle) = &self.thread_handle {
-            let mut abort = self.abort.write().unwrap();
-            *abort = true;
+            if let Ok(mut abort) = self.abort.write() {
+                *abort = true;
+            } else {
+                // Don't use log in drop because not sure if always still initialized
+                eprintln!("Unable to write abort signal to decoding thread.");
+            }
             handle.thread().unpark();
         }
     }
@@ -32,16 +53,17 @@ impl Drop for ImageRsOther {
 
 impl ImageRsOther {
     fn reader(
-        path: &Path,
+        file: &gio::File,
         format: image_rs::ImageFormat,
-    ) -> anyhow::Result<image_rs::io::Reader<BufReader<File>>> {
-        let mut reader = image_rs::io::Reader::open(path).context(i18n("Failed to open image"))?;
+    ) -> anyhow::Result<image_rs::io::Reader<Box<dyn BufReadSeek>>> {
+        let buf_read = file.to_buf_read()?;
+        let mut reader = image_rs::io::Reader::new(buf_read);
         reader.set_format(format);
         Ok(reader)
     }
 
     pub fn new(
-        path: PathBuf,
+        file: gio::File,
         format: image_rs::ImageFormat,
         updater: UpdateSender,
         tiles: Arc<ArcSwap<tiling::FrameBuffer>>,
@@ -52,14 +74,14 @@ impl ImageRsOther {
             image_rs::ImageFormat::Gif
             | image_rs::ImageFormat::WebP
             | image_rs::ImageFormat::Png => Some(Self::new_animated(
-                path,
+                file,
                 format,
                 updater,
                 tiles,
                 abort.clone(),
             )),
             _ => {
-                Self::new_static(path, format, updater, tiles);
+                Self::new_static(file, format, updater, tiles);
                 None
             }
         };
@@ -71,32 +93,57 @@ impl ImageRsOther {
     }
 
     fn new_static(
-        path: PathBuf,
+        file: gio::File,
         format: image_rs::ImageFormat,
         updater: UpdateSender,
         tiles: Arc<ArcSwap<tiling::FrameBuffer>>,
     ) {
         updater.spawn_error_handled(move || {
-            let reader = Self::reader(&path, format)?;
+            let reader = Self::reader(&file, format)?;
             let dimensions = reader
                 .into_dimensions()
                 .context("Failed to read image dimensions")?;
 
             tiles.set_original_dimensions(dimensions);
 
-            let mut reader = Self::reader(&path, format)?;
+            let (icc_profile, dynamic_image) = match format {
+                image_rs::ImageFormat::Jpeg => {
+                    let reader = file.to_buf_read()?;
+                    let mut decoder = image_rs::codecs::jpeg::JpegDecoder::new(reader)?;
+                    let _ = decoder.set_limits(image_rs::io::Limits::no_limits());
+                    let icc_profile = decoder.icc_profile();
+                    let dynamic_image = image_rs::DynamicImage::from_decoder(decoder)
+                        .context(gettext("Failed to decode image"))?;
+                    (icc_profile, dynamic_image)
+                }
+                image_rs::ImageFormat::Png => {
+                    let reader = file.to_buf_read()?;
+                    let mut decoder = image_rs::codecs::png::PngDecoder::new(reader)?;
+                    let _ = decoder.set_limits(image_rs::io::Limits::no_limits());
+                    let icc_profile = decoder.icc_profile();
+                    let dynamic_image = image_rs::DynamicImage::from_decoder(decoder)
+                        .context(gettext("Failed to decode image"))?;
+                    (icc_profile, dynamic_image)
+                }
+                _ => {
+                    let mut reader = Self::reader(&file, format)?;
+                    reader.limits(image_rs::io::Limits::no_limits());
+                    let dynamic_image =
+                        reader.decode().context(gettext("Failed to decode image"))?;
+                    (None, dynamic_image)
+                }
+            };
 
-            reader.limits(image_rs::io::Limits::no_limits());
-
-            let dynamic_image = reader.decode().context(i18n("Failed to decode image"))?;
-
-            let decoded_image = Decoded { dynamic_image };
+            let decoded_image = Decoded {
+                dynamic_image,
+                icc_profile,
+            };
 
             let tile = tiling::Tile {
                 position: (0, 0),
                 zoom_level: tiling::zoom_to_level(1.),
                 bleed: 0,
-                texture: decoded_image.into_texture(),
+                texture: decoded_image.into_texture()?,
             };
 
             tiles.push(tile);
@@ -106,7 +153,7 @@ impl ImageRsOther {
     }
 
     fn new_animated(
-        path: PathBuf,
+        file: gio::File,
         format: image_rs::ImageFormat,
         updater: UpdateSender,
         tiles: Arc<ArcSwap<tiling::FrameBuffer>>,
@@ -118,30 +165,30 @@ impl ImageRsOther {
             // We are currently decoding for each repetition of the animation
             // TODO: Check if/how that can be solved differently
             loop {
-                let file = std::fs::File::open(&path)?;
+                let buf_reader = file.to_buf_read()?;
 
                 let (frames, animated_format) = match format {
                     image_rs::ImageFormat::Gif => (
-                        image_rs::codecs::gif::GifDecoder::new(file)?.into_frames(),
+                        image_rs::codecs::gif::GifDecoder::new(buf_reader)?.into_frames(),
                         ImageFormat::AnimatedGif,
                     ),
                     image_rs::ImageFormat::WebP => {
-                        let decoder = image_rs::codecs::webp::WebPDecoder::new(file)?;
+                        let decoder = image_rs::codecs::webp::WebPDecoder::new(buf_reader)?;
                         if decoder.has_animation() {
                             (decoder.into_frames(), ImageFormat::AnimatedWebP)
                         } else {
                             // Static WebP images need a different decoder
-                            Self::new_static(path, format, updater, tiles);
+                            Self::new_static(file, format, updater, tiles);
                             return Ok(());
                         }
                     }
                     image_rs::ImageFormat::Png => {
-                        let decoder = image_rs::codecs::png::PngDecoder::new(file)?;
+                        let decoder = image_rs::codecs::png::PngDecoder::new(buf_reader)?;
                         if decoder.is_apng() {
                             (decoder.apng().into_frames(), ImageFormat::AnimatedPng)
                         } else {
                             // Static PNG images need a different decoder
-                            Self::new_static(path, format, updater, tiles);
+                            Self::new_static(file, format, updater, tiles);
                             return Ok(());
                         }
                     }
@@ -174,13 +221,16 @@ impl ImageRsOther {
 
                             let dynamic_image = image_rs::DynamicImage::from(rgb_image);
 
-                            let decoded_image = Decoded { dynamic_image };
+                            let decoded_image = Decoded {
+                                dynamic_image,
+                                icc_profile: None,
+                            };
 
                             let tile = tiling::Tile {
                                 position,
                                 zoom_level: tiling::zoom_to_level(1.),
                                 bleed: 0,
-                                texture: decoded_image.into_texture(),
+                                texture: decoded_image.into_texture()?,
                             };
 
                             tiles.push_frame(tile, dimensions, delay);
@@ -197,9 +247,13 @@ impl ImageRsOther {
                         std::thread::park();
                     }
 
-                    if *abort.read().unwrap() {
-                        log::debug!("Terminating decoder thread.");
-                        return Ok(());
+                    if let Ok(abort_state) = abort.read().as_deref() {
+                        if *abort_state {
+                            log::debug!("Terminating decoder thread.");
+                            return Ok(());
+                        }
+                    } else {
+                        bail!("Cannot read decoder thread abort instructions.");
                     }
                 }
             }
@@ -217,10 +271,11 @@ impl ImageRsOther {
 
 pub struct Decoded {
     dynamic_image: image_rs::DynamicImage,
+    icc_profile: Option<Vec<u8>>,
 }
 
 impl Decoded {
-    pub fn into_texture(self) -> gdk::Texture {
+    pub fn into_texture(self) -> anyhow::Result<gdk::Texture> {
         let (n_bytes, memory_format, layout, data) = match self.dynamic_image {
             img @ image_rs::DynamicImage::ImageLuma8(_) => {
                 let buffer = img.to_rgb8();
@@ -240,18 +295,36 @@ impl Decoded {
                     buffer.into_raw(),
                 )
             }
-            image_rs::DynamicImage::ImageRgb8(buffer) => (
-                1,
-                gdk::MemoryFormat::R8g8b8,
-                buffer.sample_layout(),
-                buffer.into_raw(),
-            ),
-            image_rs::DynamicImage::ImageRgba8(buffer) => (
-                1,
-                gdk::MemoryFormat::R8g8b8a8,
-                buffer.sample_layout(),
-                buffer.into_raw(),
-            ),
+            image_rs::DynamicImage::ImageRgb8(mut buffer) => {
+                if let Some(icc_profile) = &self.icc_profile {
+                    util::appply_icc_profile::<rgb::RGB8>(
+                        icc_profile,
+                        lcms2::PixelFormat::RGB_8,
+                        buffer.as_pixels_mut(),
+                    );
+                }
+                (
+                    1,
+                    gdk::MemoryFormat::R8g8b8,
+                    buffer.sample_layout(),
+                    buffer.into_raw(),
+                )
+            }
+            image_rs::DynamicImage::ImageRgba8(mut buffer) => {
+                if let Some(icc_profile) = &self.icc_profile {
+                    util::appply_icc_profile::<rgb::RGBA8>(
+                        icc_profile,
+                        lcms2::PixelFormat::RGBA_8,
+                        buffer.as_pixels_mut(),
+                    );
+                }
+                (
+                    1,
+                    gdk::MemoryFormat::R8g8b8a8,
+                    buffer.sample_layout(),
+                    buffer.into_raw(),
+                )
+            }
             img @ image_rs::DynamicImage::ImageLuma16(_) => {
                 let buffer = img.to_rgb16();
                 (
@@ -300,20 +373,23 @@ impl Decoded {
                     2,
                     gdk::MemoryFormat::R16g16b16a16,
                     buffer.sample_layout(),
-                    safe_transmute::transmute_vec(buffer.into_raw()).unwrap(),
+                    safe_transmute::transmute_vec(buffer.into_raw())
+                        .context("Failed to transform into 16 bit texture")?,
                 )
             }
         };
 
         let bytes = glib::Bytes::from_owned(data);
 
-        gdk::MemoryTexture::new(
+        let texture = gdk::MemoryTexture::new(
             layout.width as i32,
             layout.height as i32,
             memory_format,
             &bytes,
             layout.height_stride * n_bytes,
         )
-        .upcast()
+        .upcast();
+
+        Ok(texture)
     }
 }
