@@ -1,22 +1,39 @@
+// Copyright (c) 2023 Sophie Herold
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 ///! Decodes several image formats
-mod formats;
+pub mod formats;
 pub mod tiling;
 
 pub use formats::ImageDimensionDetails;
 
 use crate::deps::*;
-use crate::i18n::*;
 use crate::image_metadata::ImageMetadata;
+use crate::util::gettext::*;
+use crate::util::ToBufRead;
 use formats::*;
 use tiling::FrameBufferExt;
 
-use anyhow::{anyhow, bail, Context};
+use anyhow::{anyhow, bail, Context, Result};
 use arc_swap::ArcSwap;
 use futures::channel::mpsc;
 use gio::prelude::*;
 
 use std::io::Read;
-use std::path::PathBuf;
 use std::sync::Arc;
 
 pub use formats::{ImageFormat, RSVG_MAX_SIZE};
@@ -27,6 +44,7 @@ pub use formats::{ImageFormat, RSVG_MAX_SIZE};
 /// This happens for initial loading or when zoom level or viewing area changes.
 pub struct TileRequest {
     pub viewport: graphene::Rect,
+    pub area: graphene::Rect,
     pub zoom: f64,
 }
 
@@ -77,6 +95,7 @@ impl UpdateSender {
 #[derive(Debug)]
 pub struct Decoder {
     decoder: FormatDecoder,
+    update_sender: UpdateSender,
 }
 
 #[derive(Debug)]
@@ -95,46 +114,49 @@ impl Decoder {
         file: gio::File,
         tiles: Arc<ArcSwap<tiling::FrameBuffer>>,
     ) -> anyhow::Result<(Self, mpsc::UnboundedReceiver<DecoderUpdate>)> {
-        let path = file.path().context("Need a file path")?;
         let (sender, receiver) = mpsc::unbounded();
 
         let update_sender = UpdateSender { sender };
         tiles.set_update_sender(update_sender.clone());
 
-        let decoder = gio::spawn_blocking(move || Self::format_decoder(update_sender, path, tiles))
-            .await
-            .map_err(|_| anyhow!("Constructing the FormatDecoder failed unexpectedly"))??;
+        let decoder = gio::spawn_blocking(
+            glib::clone!(@strong update_sender => move || Self::format_decoder(
+                update_sender,
+                file,
+                tiles
+            )),
+        )
+        .await
+        .map_err(|_| anyhow!("Constructing the FormatDecoder failed unexpectedly"))??;
 
-        Ok((Self { decoder }, receiver))
+        Ok((
+            Self {
+                decoder,
+                update_sender,
+            },
+            receiver,
+        ))
     }
 
     fn format_decoder(
         update_sender: UpdateSender,
-        path: PathBuf,
+        file: gio::File,
         tiles: Arc<ArcSwap<tiling::FrameBuffer>>,
     ) -> anyhow::Result<FormatDecoder> {
-        update_sender.send(DecoderUpdate::Metadata(ImageMetadata::load(&path)));
+        update_sender.send(DecoderUpdate::Metadata(ImageMetadata::load(&file)));
 
-        let mut buf = Vec::new();
-        let file = std::fs::File::open(&path).context(i18n("Could not open image"))?;
-        file.take(64)
-            .read_to_end(&mut buf)
-            .context(i18n("Could not read image"))?;
+        let format = Self::guess_format(&file)?;
 
-        // Try magic bytes first and than file name extension
-        let format =
-            image_rs::guess_format(&buf).or_else(|_| image_rs::ImageFormat::from_path(&path));
-
-        if let Ok(format) = format {
+        if let Some(format) = format {
             match format {
                 image_rs::ImageFormat::Avif => {
                     update_sender.send(DecoderUpdate::Format(ImageFormat::Heif));
-                    return Ok(FormatDecoder::Heif(Heif::new(path, update_sender, tiles)));
+                    return Ok(FormatDecoder::Heif(Heif::new(file, update_sender, tiles)));
                 }
                 format => {
                     update_sender.send(DecoderUpdate::Format(ImageFormat::ImageRs(format)));
                     return Ok(FormatDecoder::ImageRsOther(ImageRsOther::new(
-                        path,
+                        file,
                         format,
                         update_sender,
                         tiles,
@@ -142,14 +164,14 @@ impl Decoder {
                 }
             }
         } else {
-            let file_info = gio::File::for_path(&path)
+            let file_info = file
                 .query_info(
                     gio::FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                     gio::FileQueryInfoFlags::NONE,
                     gio::Cancellable::NONE,
                 )
                 .context("Could not read content type information")?;
-            if let Some(content_type) = file_info
+            if let Some(mime_type) = file_info
                 .content_type()
                 .as_ref()
                 .and_then(|x| gio::content_type_get_mime_type(x))
@@ -157,27 +179,29 @@ impl Decoder {
                 // Known things we want to match here are
                 // - image/svg+xml
                 // - image/svg+xml-compressed
-                if content_type.split('+').next() == Some("image/svg") {
+                if mime_type.split('+').next() == Some("image/svg") {
                     update_sender.send(DecoderUpdate::Format(ImageFormat::Svg));
-                    return Ok(FormatDecoder::Svg(Svg::new(path, update_sender, tiles)));
-                } else if ["image/avif", "image/heif", "image/heic"]
-                    .contains(&content_type.as_str())
-                {
+                    return Ok(FormatDecoder::Svg(Svg::new(file, update_sender, tiles)));
+                } else if ["image/avif", "image/heif", "image/heic"].contains(&mime_type.as_str()) {
                     update_sender.send(DecoderUpdate::Format(ImageFormat::Heif));
-                    return Ok(FormatDecoder::Heif(Heif::new(path, update_sender, tiles)));
+                    return Ok(FormatDecoder::Heif(Heif::new(file, update_sender, tiles)));
                 } else {
-                    bail!(i18n_f("Unknown image format: {}", &[content_type.as_str()]));
+                    bail!(gettext_f("Unknown image format: {}", &[mime_type.as_str()]));
                 }
             }
         }
 
-        bail!(i18n("Unknown image format"))
+        bail!(gettext("Unknown image format"))
     }
 
     /// Request missing tiles
     pub fn request(&self, tile_request: TileRequest) {
         match &self.decoder {
-            FormatDecoder::Svg(svg) => svg.request(tile_request, self).unwrap(),
+            FormatDecoder::Svg(svg) => {
+                if let Err(err) = svg.request(tile_request) {
+                    self.update_sender.send(DecoderUpdate::Error(err));
+                }
+            }
             FormatDecoder::Heif(_) => {}
             _ => {}
         };
@@ -190,4 +214,25 @@ impl Decoder {
             log::error!("Trying to fill frame buffer for decoder without animation support");
         }
     }
+
+    fn guess_format(file: &gio::File) -> anyhow::Result<Option<image_rs::ImageFormat>> {
+        let mut buf = Vec::new();
+        let fs_file = file
+            .to_buf_read()
+            .context(gettext("Could not open image"))?;
+        fs_file
+            .take(64)
+            .read_to_end(&mut buf)
+            .context(gettext("Could not read image"))?;
+
+        // Try magic bytes first and than file name extension
+
+        if let Ok(format) = image_rs::guess_format(&buf) {
+            Ok(Some(format))
+        } else if let Some(basename) = file.basename() {
+            Ok(image_rs::ImageFormat::from_path(basename).ok())
+        } else {
+            Ok(None)
+        }
+    }
 }
