@@ -1,5 +1,7 @@
-//
-// Copyright 2021 Christopher Davis <christopherdavis@gnome.org>
+// Copyright (c) 2021-2022 Christopher Davis
+// Copyright (c) 2022-2023 Sophie Herold
+// Copyright (c) 2022 Maximiliano Sandoval R
+// Copyright (c) 2023 Lubosz Sarnecki
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,7 +33,6 @@ use crate::deps::*;
 
 use crate::decoder::tiling::FrameBufferExt;
 use crate::decoder::{self, tiling, Decoder, DecoderUpdate};
-use crate::i18n::i18n;
 use crate::image_metadata::LpImageMetadata;
 use crate::util::Gesture;
 
@@ -43,7 +44,6 @@ use once_cell::sync::Lazy;
 use once_cell::unsync::OnceCell;
 
 use std::cell::{Cell, RefCell};
-use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
 /// Default background color around images and behind transparent images
@@ -55,12 +55,13 @@ static BACKGROUND_COLOR_DEFAULT: Lazy<gdk::RGBA> =
 static BACKGROUND_COLOR_ALTERNATE: Lazy<gdk::RGBA> =
     Lazy::new(|| gdk::RGBA::new(232. / 255., 231. / 255., 230. / 255., 1.));
 
-/// Consider pixels with less than 70% opacity as being transparent
-static BACKGROUND_GUESS_TRANSPRAENT_PIXEL_THRESHOLD: u8 = (0.70 * 255.) as u8;
+static BACKGROUND_COLOR_DEFAULT_LIGHT_MODE: Lazy<gdk::RGBA> =
+    Lazy::new(|| gdk::RGBA::new(250. / 255., 250. / 255., 250. / 255., 1.));
+static BACKGROUND_COLOR_ALTERNATE_LIGHT_MODE: Lazy<gdk::RGBA> =
+    Lazy::new(|| gdk::RGBA::new(103. / 255., 101. / 255., 110. / 255., 1.));
+
 /// Consider 3.5:1 contrast and worse to be bad contrast for a pixel
 static BACKGROUND_GUESS_LOW_CONTRAST_RATIO: f32 = 3.5;
-/// Consider images with more than 10% transparent pixels as transparent
-static BACKGROUND_GUESS_TRANSPRAENT_IMAGE_THRESHOLD: f64 = 0.10;
 /// Consider transparent images with more than 90% pixels bad contrast as bad contrast
 ///
 /// Bad contrast image will use the `BACKGROUND_COLOR_ALTERNATE`.
@@ -107,7 +108,6 @@ mod imp {
     #[derive(Debug, Default)]
     pub struct LpImage {
         pub(super) file: RefCell<Option<gio::File>>,
-        pub(super) path: RefCell<Option<PathBuf>>,
         pub(super) is_deleted: Cell<bool>,
         /// Track changes to this image
         pub(super) file_monitor: RefCell<Option<gio::FileMonitor>>,
@@ -194,9 +194,6 @@ mod imp {
                     glib::ParamSpecObject::builder::<gio::File>("file")
                         .read_only()
                         .build(),
-                    glib::ParamSpecVariant::builder("path", glib::VariantTy::BYTE_STRING)
-                        .read_only()
-                        .build(),
                     glib::ParamSpecBoolean::builder("is-deleted")
                         .read_only()
                         .build(),
@@ -242,7 +239,6 @@ mod imp {
             let obj = self.obj();
             match pspec.name() {
                 "file" => obj.file().to_value(),
-                "path" => obj.path().to_variant().to_value(),
                 "is-deleted" => obj.is_deleted().to_value(),
                 "is-loaded" => obj.is_loaded().to_value(),
                 "error" => obj.error().to_value(),
@@ -295,6 +291,16 @@ mod imp {
             obj.connect_scale_factor_notify(|obj| {
                 obj.queue_resize();
             });
+
+            adw::StyleManager::default().connect_dark_notify(glib::clone!(@weak obj => move |_| {
+                spawn(async move {
+                    let color = obj.background_color_guess().await;
+                    obj.set_background_color(color);
+                    if obj.is_mapped() {
+                        obj.queue_draw();
+                    }
+                });
+            }));
         }
 
         fn dispose(&self) {
@@ -362,23 +368,11 @@ mod imp {
                 let zoom =
                     obj.imp().zoom_target.get() * zoom_factor;
 
-                if zoom > obj.zoom_level_best_fit() {
-                    obj.set_best_fit(false);
                     if animated {
                         obj.zoom_to(zoom);
                     } else {
-                        obj.set_zoom(zoom);
-                        obj.imp().zoom_target.set(zoom);
+                        obj.zoom_to_full(zoom, false, false);
                     }
-                } else {
-                    obj.set_best_fit(true);
-                    if animated {
-                        obj.zoom_to(obj.zoom_level_best_fit());
-                    } else {
-                        obj.set_zoom(obj.zoom_level_best_fit());
-                        obj.imp().zoom_target.set(obj.zoom_level_best_fit());
-                    }
-                }
 
                 // do not propagate event to scrolled window
                 gtk::Inhibit(true)
@@ -418,10 +412,16 @@ mod imp {
             );
 
             // Drag for moving image around
-            let drag_gesture = gtk::GestureDrag::new();
+            let drag_gesture = gtk::GestureDrag::builder().button(0).build();
             obj.add_controller(drag_gesture.clone());
 
             drag_gesture.connect_drag_begin(glib::clone!(@weak obj => move |gesture, _, _| {
+                // Allow only left and middle button
+                if ![1, 2].contains(&gesture.current_button()) {
+                    gesture.set_state(gtk::EventSequenceState::Denied);
+                    return;
+                }
+
                 if obj.is_hscrollable() || obj.is_vscrollable() {
                     obj.set_cursor(gdk::Cursor::from_name("grabbing", None).as_ref());
                     obj.imp().last_drag_value.set(Some((0., 0.)));
@@ -645,7 +645,12 @@ mod imp {
             ));
 
             // Apply rotation and mirroring
-            widget.snapshot_rotate_mirror(snapshot, widget.rotation() as f32, widget.mirrored());
+            widget.snapshot_rotate_mirror(
+                snapshot,
+                widget.rotation() as f32,
+                widget.mirrored(),
+                applicable_zoom,
+            );
 
             // Add texture(s)
             self.frame_buffer
@@ -661,73 +666,74 @@ mod imp {
             if image_width > 0 && image_height > 0 {
                 if let Some(display) = gdk::Display::default() {
                     if let Some(native) = self.obj().native() {
-                        let hidpi_scale = self.obj().scale_factor() as f64;
+                        if let Some(monitor) = display.monitor_at_surface(&native.surface()) {
+                            let hidpi_scale = self.obj().scale_factor() as f64;
 
-                        // get monitor information
-                        // TODO: Get rid of unwrap
-                        let monitor = display.monitor_at_surface(&native.surface()).unwrap();
-                        let monitor_geometry = monitor.geometry();
-                        // TODO: Per documentation those dimensions should not be physical pixels.
-                        // But on Wayland they are physical pixels and on X11 not.
-                        // Taking the version that works on Wayland for now.
-                        // <https://gitlab.gnome.org/GNOME/gtk/-/issues/5391>
-                        let monitor_width = monitor_geometry.width() as f64 - 40.;
-                        let monitor_height = monitor_geometry.height() as f64 - 60.;
+                            let monitor_geometry = monitor.geometry();
+                            // TODO: Per documentation those dimensions should not be physical pixels.
+                            // But on Wayland they are physical pixels and on X11 not.
+                            // Taking the version that works on Wayland for now.
+                            // <https://gitlab.gnome.org/GNOME/gtk/-/issues/5391>
+                            let monitor_width = monitor_geometry.width() as f64 - 40.;
+                            let monitor_height = monitor_geometry.height() as f64 - 60.;
 
-                        // areas
-                        let monitor_area = monitor_width * monitor_height;
-                        let image_area = image_width as f64 * image_height as f64;
+                            // areas
+                            let monitor_area = monitor_width * monitor_height;
+                            let image_area = image_width as f64 * image_height as f64;
 
-                        let occupy_area_factor = if monitor_area < 1024. * 768. {
-                            // for small monitors occupy 80% of the area
-                            0.8
-                        } else {
-                            // for large monitors occupy 30% of the area
-                            0.3
-                        };
+                            let occupy_area_factor = if monitor_area < 1024. * 768. {
+                                // for small monitors occupy 80% of the area
+                                0.8
+                            } else {
+                                // for large monitors occupy 30% of the area
+                                0.3
+                            };
 
-                        // factor for width and height that will achieve the desired area occupation
-                        // derived from:
-                        // monitor_area * occupy_area_factor ==
-                        //   (image_width * size_scale) * (image_height * size_scale)
-                        let size_scale = f64::sqrt(monitor_area / image_area * occupy_area_factor);
-                        // ensure that we never increase image size
-                        let target_scale = f64::min(1.0, size_scale);
-                        let mut nat_width = image_width as f64 * target_scale;
-                        let mut nat_height = image_height as f64 * target_scale;
+                            // factor for width and height that will achieve the desired area occupation
+                            // derived from:
+                            // monitor_area * occupy_area_factor ==
+                            //   (image_width * size_scale) * (image_height * size_scale)
+                            let size_scale =
+                                f64::sqrt(monitor_area / image_area * occupy_area_factor);
+                            // ensure that we never increase image size
+                            let target_scale = f64::min(1.0, size_scale);
+                            let mut nat_width = image_width as f64 * target_scale;
+                            let mut nat_height = image_height as f64 * target_scale;
 
-                        // scale down if targeted occupation does not fit in one direction
-                        if nat_width > monitor_width {
-                            nat_width = monitor_width;
-                            nat_height = nat_height * monitor_width / nat_width;
+                            // scale down if targeted occupation does not fit in one direction
+                            if nat_width > monitor_width {
+                                nat_width = monitor_width;
+                                nat_height = nat_height * monitor_width / nat_width;
+                            }
+
+                            // same for other direction
+                            if nat_height > monitor_height {
+                                nat_height = monitor_height;
+                                nat_width = nat_width * monitor_height / nat_height;
+                            }
+
+                            let size = match orientation {
+                                gtk::Orientation::Horizontal => (nat_width / hidpi_scale).round(),
+                                gtk::Orientation::Vertical => (nat_height / hidpi_scale).round(),
+                                _ => unreachable!(),
+                            };
+
+                            return (0, size as i32, -1, -1);
                         }
-
-                        // same for other direction
-                        if nat_height > monitor_height {
-                            nat_height = monitor_height;
-                            nat_width = nat_width * monitor_height / nat_height;
-                        }
-
-                        let size = match orientation {
-                            gtk::Orientation::Horizontal => (nat_width / hidpi_scale).round(),
-                            gtk::Orientation::Vertical => (nat_height / hidpi_scale).round(),
-                            _ => unreachable!(),
-                        };
-
-                        return (0, size as i32, -1, -1);
                     }
                 }
             }
 
             // fallback if monitor size or image size is not known:
             // use original image size and hope for the best
-            // TODO: We could use a small default monitor size estimate instead
             let size = match orientation {
                 gtk::Orientation::Horizontal => image_width,
                 gtk::Orientation::Vertical => image_height,
                 _ => unreachable!(),
             };
 
+            log::warn!("Not enough information available to calculate fitting window size");
+
             (0, size, -1, -1)
         }
     }
@@ -742,19 +748,16 @@ glib::wrapper! {
 }
 
 impl LpImage {
-    pub async fn load(&self, path: &Path) {
-        let path = path.to_path_buf();
-        let file = gio::File::for_path(&path);
-        self.set_file(&file);
+    pub async fn load(&self, file: &gio::File) {
+        log::debug!("Loading file {}", file.uri());
 
-        if !path.is_file() {
-            self.set_error(anyhow::Error::msg(i18n("File does not exist")));
-            return;
-        }
-
-        log::debug!("Loading file {path:?}");
+        self.set_file(file);
 
         let tiles = &self.imp().frame_buffer;
+        // Delete all stored textures for reloads
+        tiles.reset();
+        // Reset background color for reloads
+        self.set_background_color(None);
 
         let (decoder, mut decoder_update) = match Decoder::new(file.clone(), tiles.clone()).await {
             Ok(x) => x,
@@ -807,7 +810,7 @@ impl LpImage {
                 imp.frame_buffer.rcu(|tiles| {
                     let mut new_tiles = (**tiles).clone();
                     // TODO: Use an area larger than the viewport
-                    new_tiles.cleanup(imp.zoom_target.get(), self.viewport());
+                    new_tiles.cleanup(imp.zoom_target.get(), self.preload_area());
                     new_tiles
                 });
                 if imp.background_color.borrow().is_none() {
@@ -921,16 +924,10 @@ impl LpImage {
         self.imp().file.borrow().clone()
     }
 
-    pub fn path(&self) -> Option<PathBuf> {
-        self.imp().path.borrow().clone()
-    }
-
     pub(super) fn set_file(&self, file: &gio::File) {
         let imp = self.imp();
 
         imp.file.replace(Some(file.clone()));
-        imp.path.replace(file.path());
-        self.notify("path");
 
         let monitor = file.monitor_file(gio::FileMonitorFlags::WATCH_MOVES, gio::Cancellable::NONE);
         if let Ok(m) = &monitor {
@@ -954,7 +951,7 @@ impl LpImage {
         match event {
             gio::FileMonitorEvent::Renamed => {
                 if let Some(file) = file_b {
-                    log::debug!("Moved to {:?}", file.path());
+                    log::debug!("Moved to {}", file.uri());
                     self.set_file(file);
                 }
             }
@@ -963,7 +960,7 @@ impl LpImage {
                 let file = file_a.clone();
                 // TODO: error handling is missing
                 spawn(async move {
-                    obj.load(&file.path().unwrap()).await;
+                    obj.load(&file).await;
                 });
             }
             gio::FileMonitorEvent::Deleted
@@ -978,7 +975,7 @@ impl LpImage {
 
     /// Returns a thumbnail of the displated image
     pub fn thumbnail(&self) -> Option<gdk::Paintable> {
-        let (width, height) = self.original_dimensions();
+        let (width, height) = self.image_size();
         let long_side = i32::max(width, height);
         let orientation = self.metadata().orientation();
 
@@ -995,6 +992,7 @@ impl LpImage {
             &snapshot,
             -orientation.rotation as f32,
             orientation.mirrored,
+            scale as f64,
         );
 
         self.imp()
@@ -1005,6 +1003,35 @@ impl LpImage {
         snapshot.to_paintable(None)
     }
 
+    pub fn print_data(&self, scale: f64) -> Option<gdk::Texture> {
+        let orientation = self.metadata().orientation();
+
+        let render_options = tiling::RenderOptions {
+            scaling_filter: gsk::ScalingFilter::Trilinear,
+            background_color: None,
+            scale_factor: 1,
+        };
+
+        let snapshot = gtk::Snapshot::new();
+
+        self.snapshot_rotate_mirror(
+            &snapshot,
+            -orientation.rotation as f32,
+            orientation.mirrored,
+            scale,
+        );
+
+        self.imp()
+            .frame_buffer
+            .load()
+            .add_to_snapshot(&snapshot, scale, &render_options);
+
+        let node = snapshot.to_node()?;
+        let renderer = self.root()?.renderer();
+
+        Some(renderer.render_texture(&node, None))
+    }
+
     fn mirrored(&self) -> bool {
         self.imp().mirrored.get()
     }
@@ -1212,7 +1239,8 @@ impl LpImage {
         if let Some(decoder) = self.imp().decoder.borrow().as_ref() {
             if self.zoom_animation().state() != adw::AnimationState::Playing {
                 decoder.request(crate::decoder::TileRequest {
-                    viewport: self.viewport(),
+                    viewport: self.viewport().inset_r(-3., -3.),
+                    area: self.preload_area(),
                     zoom: self.imp().zoom_target.get(),
                 });
             }
@@ -1230,6 +1258,11 @@ impl LpImage {
         graphene::Rect::new(x, y, width, height)
     }
 
+    fn preload_area(&self) -> graphene::Rect {
+        let viewport = self.viewport();
+        viewport.inset_r(-viewport.width() / 3., -viewport.height() / 3.)
+    }
+
     /// Animation that makes larger zoom steps (from buttons etc) look smooth
     fn zoom_animation(&self) -> &adw::TimedAnimation {
         self.imp().zoom_animation.get_or_init(|| {
@@ -1305,7 +1338,18 @@ impl LpImage {
     }
 
     /// Zoom to specific level with animation
-    pub fn zoom_to(&self, mut zoom: f64) {
+    pub fn zoom_to(&self, zoom: f64) {
+        self.zoom_to_full(zoom, true, true);
+    }
+
+    /// Zoom to specific level with animation not snapping to best-fit
+    ///
+    /// Used for zooming to 100% or 200%
+    pub fn zoom_to_exact(&self, zoom: f64) {
+        self.zoom_to_full(zoom, true, false);
+    }
+
+    fn zoom_to_full(&self, mut zoom: f64, animated: bool, snap_best_fit: bool) {
         let max_zoom = self.max_zoom();
         if zoom >= max_zoom {
             zoom = max_zoom;
@@ -1314,11 +1358,14 @@ impl LpImage {
             self.set_max_zoom(false);
         }
 
-        // If image is only 1/4 of a zoom step away from best-fit, also
-        // activate best-fit. This avoids bugs with floating point precision
-        // and removes awkward minimal zoom steps.
-        let extended_best_fit_threshold =
-            self.zoom_level_best_fit() * (1. + (ZOOM_FACTOR_BUTTON - 1.) / 4.);
+        let extended_best_fit_threshold = if snap_best_fit {
+            // If image is only 1/4 of a zoom step away from best-fit, also
+            // activate best-fit. This avoids bugs with floating point precision
+            // and removes awkward minimal zoom steps.
+            self.zoom_level_best_fit() * (1. + (ZOOM_FACTOR_BUTTON - 1.) / 4.)
+        } else {
+            self.zoom_level_best_fit()
+        };
 
         if zoom <= extended_best_fit_threshold {
             zoom = self.zoom_level_best_fit();
@@ -1337,26 +1384,30 @@ impl LpImage {
             return;
         }
 
-        // wild code
-        let current_hborder = self.widget_width() - self.image_displayed_width();
-        let target_hborder = self.widget_width() - self.image_size().0 as f64 * zoom;
+        if animated {
+            // wild code
+            let current_hborder = self.widget_width() - self.image_displayed_width();
+            let target_hborder = self.widget_width() - self.image_size().0 as f64 * zoom;
 
-        self.imp()
-            .zoom_hscrollbar_transition
-            .set(current_hborder.signum() != target_hborder.signum() && current_hborder != 0.);
+            self.imp()
+                .zoom_hscrollbar_transition
+                .set(current_hborder.signum() != target_hborder.signum() && current_hborder != 0.);
 
-        let current_vborder = self.widget_height() - self.image_displayed_height();
-        let target_vborder = self.widget_height() - self.image_size().1 as f64 * zoom;
+            let current_vborder = self.widget_height() - self.image_displayed_height();
+            let target_vborder = self.widget_height() - self.image_size().1 as f64 * zoom;
 
-        self.imp()
-            .zoom_hscrollbar_transition
-            .set(current_vborder.signum() != target_vborder.signum() && current_vborder != 0.);
+            self.imp()
+                .zoom_hscrollbar_transition
+                .set(current_vborder.signum() != target_vborder.signum() && current_vborder != 0.);
 
-        let animation = self.zoom_animation();
+            let animation = self.zoom_animation();
 
-        animation.set_value_from(self.zoom());
-        animation.set_value_to(zoom);
-        animation.play();
+            animation.set_value_from(self.zoom());
+            animation.set_value_to(zoom);
+            animation.play();
+        } else {
+            self.set_zoom(zoom)
+        }
     }
 
     /// Image size of original image with EXIF rotation applied
@@ -1384,19 +1435,27 @@ impl LpImage {
     /// represent the actual size. The size returned might well be
     /// larger than what can actually be displayed within the widget.
     pub fn image_displayed_width(&self) -> f64 {
-        let (width, height) = self.original_dimensions();
-
-        let rotated = self.rotation().to_radians().sin().abs();
-
-        ((1. - rotated) * width as f64 + rotated * height as f64) * self.applicable_zoom()
+        self.image_width(self.applicable_zoom())
     }
 
     pub fn image_displayed_height(&self) -> f64 {
+        self.image_height(self.applicable_zoom())
+    }
+
+    pub fn image_width(&self, zoom: f64) -> f64 {
         let (width, height) = self.original_dimensions();
 
         let rotated = self.rotation().to_radians().sin().abs();
 
-        ((1. - rotated) * height as f64 + rotated * width as f64) * self.applicable_zoom()
+        ((1. - rotated) * width as f64 + rotated * height as f64) * zoom
+    }
+
+    pub fn image_height(&self, zoom: f64) -> f64 {
+        let (width, height) = self.original_dimensions();
+
+        let rotated = self.rotation().to_radians().sin().abs();
+
+        ((1. - rotated) * height as f64 + rotated * width as f64) * zoom
     }
 
     /// Stepwise scrolls inside an image when zoomed in
@@ -1540,20 +1599,25 @@ impl LpImage {
     ///
     /// After the operation the image is positioned such that it's origin
     /// is a `(0, 0)` again.
-    fn snapshot_rotate_mirror(&self, snapshot: &gtk::Snapshot, rotation: f32, mirrored: bool) {
+    fn snapshot_rotate_mirror(
+        &self,
+        snapshot: &gtk::Snapshot,
+        rotation: f32,
+        mirrored: bool,
+        zoom: f64,
+    ) {
         if rotation == 0. && !mirrored {
             return;
         }
 
-        let zoom = self.applicable_zoom() as f32;
         let (original_width, original_height) = self.original_dimensions();
-        let displayed_width = self.image_displayed_width() as f32;
-        let displayed_height = self.image_displayed_height() as f32;
+        let width = self.image_width(zoom) as f32;
+        let height = self.image_height(zoom) as f32;
 
         // Put image origin at (0, 0) again with rotation
         snapshot.translate(&graphene::Point::new(
-            self.round_f32(displayed_width / 2.),
-            self.round_f32(displayed_height / 2.),
+            self.round_f32(width / 2.),
+            self.round_f32(height / 2.),
         ));
 
         // Apply the transformations from properties
@@ -1566,8 +1630,8 @@ impl LpImage {
         // Needed for rotating around the center of the image, and
         // mirroring the image does not put it to a completely different position.
         snapshot.translate(&graphene::Point::new(
-            -self.round_f32(original_width as f32 * zoom / 2.),
-            -self.round_f32(original_height as f32 * zoom / 2.),
+            -self.round_f32(original_width as f32 * zoom as f32 / 2.),
+            -self.round_f32(original_height as f32 * zoom as f32 / 2.),
         ));
     }
 
@@ -1633,13 +1697,29 @@ impl LpImage {
     ///
     /// Returns the default color if no one has been guessed yet
     pub fn background_color(&self) -> gdk::RGBA {
-        (*self.imp().background_color.borrow()).unwrap_or(*BACKGROUND_COLOR_DEFAULT)
+        (*self.imp().background_color.borrow()).unwrap_or_else(Self::default_background_color)
     }
 
     pub fn set_background_color(&self, color: Option<gdk::RGBA>) {
         self.imp().background_color.replace(color);
     }
 
+    pub fn default_background_color() -> gdk::RGBA {
+        if adw::StyleManager::default().is_dark() {
+            *BACKGROUND_COLOR_DEFAULT
+        } else {
+            *BACKGROUND_COLOR_DEFAULT_LIGHT_MODE
+        }
+    }
+
+    pub fn alternate_background_color() -> gdk::RGBA {
+        if adw::StyleManager::default().is_dark() {
+            *BACKGROUND_COLOR_ALTERNATE
+        } else {
+            *BACKGROUND_COLOR_ALTERNATE_LIGHT_MODE
+        }
+    }
+
     /// Returns a background color that should give suitable contrast with transparent images
     ///
     /// For non-transparent images this always returns `BACKGROUND_COLOR_DEFAULT`
@@ -1650,7 +1730,7 @@ impl LpImage {
             .map_or(true, |x| x.is_potentially_transparent())
         {
             log::debug!("This format does not support transparency");
-            return Some(*BACKGROUND_COLOR_DEFAULT);
+            return Some(Self::default_background_color());
         }
 
         let (width, height) = self.original_dimensions();
@@ -1683,10 +1763,14 @@ impl LpImage {
         downloader.set_format(gdk::MemoryFormat::R8g8b8a8);
         let (bytes, stride) = downloader.download_bytes();
 
+        // Get here because only available in main thread
+        let alternate_color = Self::alternate_background_color();
+        let default_color = Self::default_background_color();
+
         gio::spawn_blocking(move || {
             let mut bytes_iter = bytes.iter();
             // Number of transparent pixels
-            let mut transparent = 0;
+            let mut completely_transparent = 0;
             // Number of non-transparent pixels with bad contrast
             let mut bad_contrast = 0;
             'img: loop {
@@ -1697,8 +1781,8 @@ impl LpImage {
                     let Some(a) = bytes_iter.next() else { break 'img; };
 
                     // 70% transparency
-                    if *a < BACKGROUND_GUESS_TRANSPRAENT_PIXEL_THRESHOLD {
-                        transparent += 1;
+                    if *a == 0 {
+                        completely_transparent += 1;
                     } else {
                         let fg = gdk::RGBA::new(
                             *r as f32 / 255.,
@@ -1706,7 +1790,7 @@ impl LpImage {
                             *b as f32 / 255.,
                             *a as f32 / 255.,
                         );
-                        let contrast = crate::util::contrast_ratio(&BACKGROUND_COLOR_DEFAULT, &fg);
+                        let contrast = crate::util::contrast_ratio(&default_color, &fg);
 
                         if contrast < BACKGROUND_GUESS_LOW_CONTRAST_RATIO {
                             bad_contrast += 1;
@@ -1723,15 +1807,16 @@ impl LpImage {
 
             let n_pixels = texture.width() * texture.height();
 
-            let part_transparent = transparent as f64 / n_pixels as f64;
-            let part_bad_contrast = bad_contrast as f64 / (n_pixels as f64 - transparent as f64);
-
-            if part_transparent > BACKGROUND_GUESS_TRANSPRAENT_IMAGE_THRESHOLD
-                && part_bad_contrast > BACKGROUND_GUESS_LOW_CONTRAST_TRHESHOLD
-            {
-                Some(*BACKGROUND_COLOR_ALTERNATE)
+            let part_bad_contrast = if completely_transparent < n_pixels {
+                bad_contrast as f64 / (n_pixels as f64 - completely_transparent as f64)
             } else {
-                Some(*BACKGROUND_COLOR_DEFAULT)
+                1.
+            };
+
+            if part_bad_contrast > BACKGROUND_GUESS_LOW_CONTRAST_TRHESHOLD {
+                Some(alternate_color)
+            } else {
+                Some(default_color)
             }
         })
         .await
