@@ -1,6 +1,8 @@
-// image_view.rs
-//
-// Copyright 2020 Christopher Davis <christopherdavis@gnome.org>
+// Copyright (c) 2020-2023 Christopher Davis
+// Copyright (c) 2022-2023 Sophie Herold
+// Copyright (c) 2022 Elton A Rodrigues
+// Copyright (c) 2022 Maximiliano Sandoval R
+// Copyright (c) 2023 Philipp Kiemle
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,9 +29,9 @@
 
 use crate::deps::*;
 use crate::file_model::LpFileModel;
-use crate::i18n::*;
+use crate::util::gettext::*;
 use crate::util::{Direction, Position};
-use crate::widgets::{LpImage, LpImagePage, LpSlidingView};
+use crate::widgets::{LpImage, LpImagePage, LpPrint, LpSlidingView};
 
 use crate::util::spawn;
 use adw::prelude::*;
@@ -44,7 +46,6 @@ use gtk::CompositeTemplate;
 use once_cell::sync::Lazy;
 
 use std::cell::{Cell, RefCell};
-use std::path::{Path, PathBuf};
 
 // The number of pages we want to buffer
 // on either side of the current page.
@@ -64,9 +65,14 @@ mod imp {
         /// overlayed controls
         #[template_child]
         pub(super) controls_box_start: TemplateChild<gtk::Widget>,
+        #[template_child]
+        pub(super) controls_box_start_events: TemplateChild<gtk::EventControllerMotion>,
         /// overlayed controls
         #[template_child]
         pub(super) controls_box_end: TemplateChild<gtk::Widget>,
+        #[template_child]
+        pub(super) controls_box_end_events: TemplateChild<gtk::EventControllerMotion>,
+
         #[template_child]
         pub sliding_view: TemplateChild<LpSlidingView>,
 
@@ -139,7 +145,7 @@ mod imp {
 
             let view = &*obj;
             signal_group.connect_closure(
-                "notify::path",
+                "notify::file",
                 true,
                 glib::closure_local!(@watch view => move |_: LpImage, _: glib::ParamSpec| {
                     view.current_image_path_changed();
@@ -228,15 +234,35 @@ glib::wrapper! {
 
 #[gtk::template_callbacks]
 impl LpImageView {
-    pub fn set_image_from_path(&self, path: &Path) {
+    pub fn set_image_from_file(&self, file: gio::File) {
         // Add image to recently used file. Does not work in Flatpaks:
         // <https://github.com/flatpak/xdg-desktop-portal/issues/215>
-        gtk::RecentManager::default().add_item(&gio::File::for_path(path).uri());
+        gtk::RecentManager::default().add_item(&file.uri());
 
-        if let Err(err) = self.load_path(path) {
-            log::error!("Failed to load path: {err}");
-            self.activate_action("win.show-toast", Some(&(err.to_string(), 1).to_variant()))
-                .unwrap();
+        self.load_file(file);
+    }
+
+    pub fn set_images_from_files(&self, files: Vec<gio::File>) {
+        let recent_manager = gtk::RecentManager::default();
+        for file in files.iter() {
+            recent_manager.add_item(&file.uri());
+        }
+        self.load_files(files);
+    }
+
+    fn load_files(&self, files: Vec<gio::File>) {
+        let sliding_view = self.sliding_view();
+
+        if let Some(first) = files.first().cloned() {
+            let model = LpFileModel::from_files(files);
+            self.set_model(model);
+            let page = LpImagePage::from_file(&first);
+
+            sliding_view.clear();
+            sliding_view.append(&page);
+            self.update_sliding_view(&first);
+        } else {
+            log::error!("File list was empty");
         }
     }
 
@@ -244,93 +270,93 @@ impl LpImageView {
     // that holds a `gio::File` for each child within the same directory as the
     // file we pass. This model will update with changes to the directory,
     // and in turn we'll update our `adw::Carousel`.
-    fn load_path(&self, path: &Path) -> anyhow::Result<()> {
-        let sliding_view = self.sliding_view();
-        let directory = path.parent().map(|x| x.to_path_buf());
+    fn load_file(&self, file: gio::File) {
+        let directory = file.parent();
 
-        let is_same_directory = directory == self.model().directory();
+        let is_same_directory = if let (Some(d1), Some(d2)) = (&directory, self.model().directory())
+        {
+            d1.equal(&d2)
+        } else {
+            false
+        };
 
         if is_same_directory {
             log::debug!("Re-using old model and navigating to the current file");
-            self.navigate_to_path(path);
-            return Ok(());
+            self.navigate_to_file(&file);
+            return;
         }
 
-        let model = LpFileModel::from_path(path);
+        let model = LpFileModel::from_file(file.clone());
         self.set_model(model);
         log::debug!("New model created");
 
-        let page = LpImagePage::from_path(path);
-
-        sliding_view.clear();
-        sliding_view.append(&page);
+        self.update_sliding_view(&file);
 
         // List other files in directory
         if let Some(directory) = directory {
-            let path = path.to_path_buf();
-            spawn(glib::clone!(@weak self as obj, @strong path => async move {
-                if let Err(err) = obj.model().load_directory(directory).await {
+            spawn(glib::clone!(@weak self as obj, @strong file => async move {
+                if let Err(err) = obj.model().load_directory(directory.clone()).await {
                     log::warn!("Failed to load directory: {err}");
                     obj.activate_action("win.show-toast", Some(&(err.to_string(), 1).to_variant()))
                         .unwrap();
                     return;
                 }
 
-                obj.update_sliding_view(&path);
+                obj.update_sliding_view(&file);
             }));
         }
-
-        Ok(())
     }
 
     /// Move forward or backwards
     pub fn navigate(&self, direction: Direction) {
-        if let Some(current_path) = self.current_path() {
-            let new_path = match direction {
-                Direction::Forward => self.model().after(&current_path),
-                Direction::Back => self.model().before(&current_path),
+        if let Some(current_file) = self.current_file() {
+            let new_file = match direction {
+                Direction::Forward => self.model().after(&current_file),
+                Direction::Back => self.model().before(&current_file),
             };
 
-            if let Some(new_path) = new_path {
-                self.scroll_sliding_view(&new_path);
+            if let Some(new_file) = new_file {
+                self.scroll_sliding_view(&new_file);
             }
         }
     }
 
     /// Jump to position
     pub fn jump(&self, position: Position) {
-        let new_path = match position {
+        let new_file = match position {
             Position::First => self.model().first(),
             Position::Last => self.model().last(),
         };
 
-        if let Some(new_path) = new_path {
-            self.navigate_to_path(&new_path);
+        if let Some(new_file) = new_file {
+            self.navigate_to_file(&new_file);
         }
     }
 
     /// Used for drag and drop
-    fn navigate_to_path(&self, new_path: &Path) {
+    fn navigate_to_file(&self, new_file: &gio::File) {
         let sliding_view = self.sliding_view();
 
-        if Some(new_path.to_path_buf()) == self.current_path() {
-            return;
+        if let Some(current_file) = self.current_file() {
+            if new_file.equal(&current_file) {
+                return;
+            }
         }
 
-        let Some(new_index) = self.model().index_of(new_path) else {
-            log::warn!("Could not navigate to file {new_path:?}");
+        let Some(new_index) = self.model().index_of(new_file) else {
+            log::warn!("Could not navigate to file {}", new_file.uri());
             return;
         };
 
         let current_index = self
-            .current_path()
+            .current_file()
             .and_then(|x| self.model().index_of(&x))
             .unwrap_or_default();
 
-        let page = if let Some(page) = sliding_view.get(new_path) {
+        let page = if let Some(page) = sliding_view.get(new_file) {
             page
         } else {
-            let page = LpImagePage::from_path(new_path);
+            let page = LpImagePage::from_file(new_file);
 
             if new_index > current_index {
                 sliding_view.append(&page);
@@ -341,34 +367,37 @@ impl LpImageView {
             page
         };
 
-        log::debug!("Scrolling to page for {new_path:?}");
+        log::debug!("Scrolling to page for {}", new_file.uri());
         self.imp().preserve_content.set(true);
         sliding_view.scroll_to(&page);
     }
 
     /// Ensures the sliding view contains the correct images
-    fn update_sliding_view(&self, current_path: &Path) {
-        log::debug!("Updating sliding_view neighbors for current path {current_path:?}");
+    fn update_sliding_view(&self, current_file: &gio::File) {
+        log::debug!(
+            "Updating sliding_view neighbors for current path {}",
+            current_file.uri()
+        );
         let sliding_view = self.sliding_view();
 
         self.imp().preserve_content.set(false);
 
         let existing = sliding_view.pages();
-        let target = self.model().files_around(current_path, BUFFER);
+        let target = self.model().files_around(current_file, BUFFER);
 
         // remove old pages
-        for (path, page) in &existing {
-            if !target.contains(path) {
+        for (uri, page) in &existing {
+            if !target.contains_key(uri) {
                 sliding_view.remove(page);
             }
         }
 
         // add missing pages or put in correct position
-        for (position, path) in target.iter().enumerate() {
-            if let Some(page) = existing.get(path) {
+        for (position, (_, file)) in target.iter().enumerate() {
+            if let Some(page) = existing.get(&file.uri()) {
                 sliding_view.move_to(page, position);
             } else {
-                sliding_view.insert(&LpImagePage::from_path(path), position);
+                sliding_view.insert(&LpImagePage::from_file(file), position);
             }
         }
 
@@ -376,9 +405,9 @@ impl LpImageView {
         self.notify("is-next-available");
     }
 
-    fn scroll_sliding_view(&self, path: &Path) {
-        let Some(current_page) = self.sliding_view().pages().remove(path) else {
-            log::error!("Current path not available in sliding_view for scrolling: {path:?}");
+    fn scroll_sliding_view(&self, file: &gio::File) {
+        let Some(current_page) = self.sliding_view().pages().remove(&file.uri()) else {
+            log::error!("Current path not available in sliding_view for scrolling: {}", file.uri());
             return;
         };
 
@@ -389,6 +418,22 @@ impl LpImageView {
         self.imp().sliding_view.clone()
     }
 
+    pub fn controls_box_start(&self) -> gtk::Widget {
+        self.imp().controls_box_start.clone()
+    }
+
+    pub fn controls_box_start_events(&self) -> gtk::EventControllerMotion {
+        self.imp().controls_box_start_events.clone()
+    }
+
+    pub fn controls_box_end(&self) -> gtk::Widget {
+        self.imp().controls_box_end.clone()
+    }
+
+    pub fn controls_box_end_events(&self) -> gtk::EventControllerMotion {
+        self.imp().controls_box_end_events.clone()
+    }
+
     fn model(&self) -> LpFileModel {
         self.imp().model.borrow().clone()
     }
@@ -402,15 +447,15 @@ impl LpImageView {
 
     /// Handle files are added or removed from directory
     fn model_content_changed_cb(&self) {
-        let Some(current_path) = self.current_path() else { return; };
+        let Some(current_file) = self.current_file() else { return; };
 
         // LpImage did not get the update yet
         // Update will be handled by current_image_path_changed
-        if !self.model().contains(&current_path) {
+        if !self.model().contains(&current_file) {
             return;
         }
 
-        self.update_sliding_view(&current_path);
+        self.update_sliding_view(&current_file);
     }
 
     /// Handle current image being moved or deleted
@@ -424,9 +469,9 @@ impl LpImageView {
             }
         }
 
-        if let Some(current_path) = self.current_path() {
-            if self.model().contains(&current_path) && !self.imp().preserve_content.get() {
-                self.update_sliding_view(&current_path);
+        if let Some(current_file) = self.current_file() {
+            if self.model().contains(&current_file) && !self.imp().preserve_content.get() {
+                self.update_sliding_view(&current_file);
             }
         }
     }
@@ -446,8 +491,11 @@ impl LpImageView {
         self.imp().sliding_view.current_page()
     }
 
-    pub fn current_path(&self) -> Option<PathBuf> {
-        self.imp().sliding_view.current_page().map(|x| x.path())
+    pub fn current_uri(&self) -> Option<glib::GString> {
+        self.imp()
+            .sliding_view
+            .current_page()
+            .map(|x| x.file().uri())
     }
 
     pub fn current_file(&self) -> Option<gio::File> {
@@ -459,8 +507,8 @@ impl LpImageView {
 
     /// Returns `true` if there is an image before the current one
     pub fn is_previous_available(&self) -> bool {
-        if let Some(path) = self.current_path() {
-            self.model().index_of(&path) != Some(0)
+        if let Some(file) = self.current_file() {
+            self.model().index_of(&file) != Some(0)
         } else {
             false
         }
@@ -468,9 +516,9 @@ impl LpImageView {
 
     /// Returns `true` if there is an image after the current one
     pub fn is_next_available(&self) -> bool {
-        if let Some(path) = self.current_path() {
+        if let Some(file) = self.current_file() {
             let model = self.model();
-            model.index_of(&path) != Some(model.n_files().saturating_sub(1))
+            model.index_of(&file) != Some(model.n_files().saturating_sub(1))
         } else {
             false
         }
@@ -488,7 +536,7 @@ impl LpImageView {
         };
 
         if !self.imp().preserve_content.get() {
-            self.update_sliding_view(&new_page.path());
+            self.update_sliding_view(&new_page.file());
         }
     }
 
@@ -499,7 +547,7 @@ impl LpImageView {
 
         if self.imp().preserve_content.get() {
             if let Some(new_page) = &current_page {
-                self.update_sliding_view(&new_page.path());
+                self.update_sliding_view(&new_page.file());
             } else {
                 log::error!("No LpImagePage");
             }
@@ -508,7 +556,7 @@ impl LpImageView {
         // Reset zoom and rotation of other pages
         if let Some(new_page) = &current_page {
             let mut other_pages = self.sliding_view().pages();
-            other_pages.remove(&new_page.path());
+            other_pages.remove(&new_page.file().uri());
             for (_, page) in other_pages {
                 let image = page.image();
                 image.reset_rotation();
@@ -539,12 +587,6 @@ impl LpImageView {
         }
     }
 
-    pub fn zoom_to(&self, level: f64) {
-        if let Some(current_page) = self.current_page() {
-            current_page.image().zoom_to(level);
-        }
-    }
-
     pub fn rotate_image(&self, angle: f64) {
         if let Some(current_page) = self.current_page() {
             current_page.image().rotate_by(angle);
@@ -552,18 +594,18 @@ impl LpImageView {
     }
 
     pub async fn set_background(&self) -> anyhow::Result<()> {
-        let background = self
-            .current_path()
-            .and_then(|p| std::fs::File::open(p).ok())
-            .context(i18n("Failed to open new background image"))?;
-        let native = self.native().expect("View should have a GtkNative");
+        let uri = self
+            .current_file()
+            .and_then(|f| url::Url::parse(f.uri().as_str()).ok())
+            .context("Invalid URL for background image")?;
+        let native = self.native().context("View should have a GtkNative")?;
         let id = WindowIdentifier::from_native(&native).await;
 
         match wallpaper::WallpaperRequest::default()
             .set_on(wallpaper::SetOn::Background)
             .show_preview(true)
             .identifier(id)
-            .build_file(&background)
+            .build_uri(&uri)
             .await
         {
             // We use `1` here because we can't pass enums directly as GVariants,
@@ -572,14 +614,21 @@ impl LpImageView {
             Ok(_) => self
                 .activate_action(
                     "win.show-toast",
-                    Some(&(i18n("Set as background."), 1).to_variant()),
+                    Some(
+                        &(
+                            // Translators: This is a toast notification, informing the user that an image has been set as background.
+                            gettext("Set as background."),
+                            1,
+                        )
+                            .to_variant(),
+                    ),
                 )
                 .unwrap(),
             Err(err) => {
                 if !matches!(err, Error::Response(ResponseError::Cancelled)) {
                     self.activate_action(
                         "win.show-toast",
-                        Some(&(i18n("Could not set background."), 1).to_variant()),
+                        Some(&(gettext("Could not set background."), 1).to_variant()),
                     )
                     .unwrap();
                 }
@@ -590,36 +639,16 @@ impl LpImageView {
     }
 
     pub fn print(&self) -> anyhow::Result<()> {
-        let operation = gtk::PrintOperation::new();
-        let path = self.current_path().context("No file")?;
-        let pb = gdk_pixbuf::Pixbuf::from_file(path)?;
+        let image = self
+            .current_image()
+            .context("No current image for printing")?;
 
-        let setup = gtk::PageSetup::default();
-        let page_size = gtk::PaperSize::new(Some(gtk::PAPER_NAME_A4));
-        setup.set_paper_size(&page_size);
-        operation.set_default_page_setup(Some(&setup));
-
-        let settings = gtk::PrintSettings::default();
-        operation.set_print_settings(Some(&settings));
-
-        operation.connect_begin_print(move |op, _| {
-            op.set_n_pages(1);
-        });
-
-        // FIXME: Rework all of this after reading https://cairographics.org/manual/cairo-Image-Surfaces.html
-        // Since I don't know cairo; See also eog-print.c
-        operation.connect_draw_page(clone!(@weak pb => move |_, ctx, _| {
-            let cr = ctx.cairo_context();
-            cr.set_source_pixbuf(&pb, 0.0, 0.0);
-            let _ = cr.paint();
-        }));
-
-        log::debug!("Running print operation...");
         let root = self.root().context("Could not get root for widget")?;
         let window = root
             .downcast_ref::<gtk::Window>()
             .context("Could not downcast to GtkWindow")?;
-        operation.run(gtk::PrintOperationAction::PrintDialog, Some(window))?;
+
+        LpPrint::new(image, window.clone(), None, None).run();
 
         Ok(())
     }
