@@ -1,6 +1,8 @@
-// window.rs
-//
-// Copyright 2020 Christopher Davis <christopherdavis@gnome.org>
+// Copyright (c) 2020-2023 Christopher Davis
+// Copyright (c) 2022-2023 Sophie Herold
+// Copyright (c) 2022 Elton A Rodrigues
+// Copyright (c) 2022 Maximiliano Sandoval R
+// Copyright (c) 2023 Lubosz Sarnecki
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +20,7 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use crate::deps::*;
-use crate::i18n::*;
+use crate::util::gettext::*;
 
 use crate::util::spawn;
 use adw::prelude::*;
@@ -26,12 +28,22 @@ use adw::subclass::prelude::*;
 use glib::clone;
 use gtk::CompositeTemplate;
 
+use once_cell::sync::OnceCell;
+
+use std::cell::RefCell;
 use std::path::{Path, PathBuf};
 
 use crate::config;
 use crate::util::{self, Direction, Position};
 use crate::widgets::{LpImage, LpImageView, LpPropertiesView};
 
+/// Animation duration for showing overlay buttons in milliseconds
+const SHOW_CONTROLS_ANIMATION_DURATION: u32 = 200;
+/// Animation duration for hiding overlay buttons in milliseconds
+const HIDE_CONTROLS_ANIMATION_DURATION: u32 = 1000;
+/// Time of inactivity after which controls will be hidden in seconds
+const HIDE_CONTROLS_IDLE_TIMEOUT: u32 = 3;
+
 mod imp {
     use super::*;
 
@@ -56,14 +68,10 @@ mod imp {
         // TemplateChild<T> wrapper, where T is the
         // object type of the template child.
         #[template_child]
-        pub(super) flap: TemplateChild<adw::Flap>,
-        #[template_child]
         pub(super) headerbar: TemplateChild<gtk::HeaderBar>,
         #[template_child]
         pub(super) properties_button: TemplateChild<gtk::ToggleButton>,
         #[template_child]
-        pub(super) menu_button: TemplateChild<gtk::MenuButton>,
-        #[template_child]
         pub(super) toast_overlay: TemplateChild<adw::ToastOverlay>,
         #[template_child]
         pub(super) stack: TemplateChild<gtk::Stack>,
@@ -75,6 +83,12 @@ mod imp {
         pub(super) properties_view: TemplateChild<LpPropertiesView>,
         #[template_child]
         pub(super) drop_target: TemplateChild<gtk::DropTarget>,
+        #[template_child]
+        pub(super) motion_controller: TemplateChild<gtk::EventControllerMotion>,
+
+        pub(super) show_controls_animation: OnceCell<adw::TimedAnimation>,
+        pub(super) hide_controls_animation: OnceCell<adw::TimedAnimation>,
+        pub(super) hide_controls_timeout: RefCell<Option<glib::SourceId>>,
     }
 
     #[glib::object_subclass]
@@ -134,8 +148,8 @@ mod imp {
                 win.zoom_in();
             });
 
-            klass.install_action("win.zoom-to", Some("d"), move |win, _, level| {
-                win.zoom_to(level.unwrap().get().unwrap());
+            klass.install_action("win.zoom-to-exact", Some("d"), move |win, _, level| {
+                win.zoom_to_exact(level.unwrap().get().unwrap());
             });
 
             klass.install_action("win.zoom-best-fit", None, move |win, _, _| {
@@ -222,7 +236,7 @@ mod imp {
             // and keeps the others usable
             gtk::WindowGroup::new().add_window(&*obj);
 
-            obj.set_actions_enabled(false);
+            obj.images_available();
 
             let current_image_signals = self.image_view.current_image_signals();
             // clone! is a macro from glib-rs that allows
@@ -302,14 +316,15 @@ mod imp {
                 }),
             );
 
+            // Make widgets visible when the focus moves
+            obj.connect_move_focus(|obj, _| {
+                obj.show_controls();
+                obj.queue_hide_controls();
+            });
+
             self.status_page
                 .set_icon_name(Some(&format!("{}-symbolic", config::APP_ID)));
 
-            // Set help overlay
-            let builder = gtk::Builder::from_resource("/org/gnome/Loupe/gtk/help_overlay.ui");
-            let help_overlay = builder.object("help_overlay").unwrap();
-            obj.set_help_overlay(Some(&help_overlay));
-
             self.drop_target.set_types(&[gdk::FileList::static_type()]);
 
             // For callbacks, you will want to reference the GTK docs on
@@ -325,38 +340,26 @@ mod imp {
                     // We've added type annotations here, and written it as `let list: gdk::FileList = ...`,
                     // but you might also see places where type arguments are used.
                     // This line could have been written as `let list = value.get::<gdk::FileList>().unwrap()`.
-                    let list: gdk::FileList = match value.get() {
-                        Ok(list) => list,
+                    let mut files = match value.get::<gdk::FileList>() {
+                        Ok(list) => list.files(),
                         Err(err) => {
                             log::error!("Issue with drop value: {err}");
                             return false;
                         }
                     };
 
-                    // TODO: Handle this like EOG and make a "directory" out of the given files
-                    let file = list.files().get(0).unwrap().clone();
-                    let info = util::query_attributes(
-                        &file,
-                        vec![
-                            gio::FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-                            gio::FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
-                        ],
-                    )
-                    .expect("Could not query file info");
-
-                    if info
-                        .content_type()
-                        .map(|t| t.to_string())
-                        .filter(|t| t.starts_with("image/"))
-                        .is_some() {
-                        obj.set_image_from_path(&file.path().unwrap());
+                    if files.len() > 1 {
+                        obj.image_view().set_images_from_files(files);
+                    } else if let Some(file) = files.pop() {
+                        obj.image_view().set_image_from_file(file);
                     } else {
-                        obj.show_toast(
-                            i18n_f("“{}” is not a valid image.", &[&info.display_name()]),
-                            adw::ToastPriority::High,
-                        );
+                        log::error!("Dropped FileList was empty");
+                        return false;
                     }
 
+                    // Maybe one day this will actually work
+                    obj.present();
+
                     true
                 }),
             );
@@ -382,16 +385,6 @@ impl LpWindow {
     }
 
     fn toggle_fullscreen(&self, fullscreen: bool) {
-        let imp = self.imp();
-
-        if fullscreen {
-            imp.headerbar.add_css_class("osd");
-            imp.properties_view.add_css_class("osd")
-        } else {
-            imp.headerbar.remove_css_class("osd");
-            imp.properties_view.remove_css_class("osd")
-        }
-
         self.set_fullscreened(fullscreen);
     }
 
@@ -403,8 +396,10 @@ impl LpWindow {
         self.imp().image_view.zoom_in();
     }
 
-    fn zoom_to(&self, level: f64) {
-        self.imp().image_view.zoom_to(level);
+    fn zoom_to_exact(&self, level: f64) {
+        if let Some(image) = self.imp().image_view.current_image() {
+            image.zoom_to_exact(level);
+        }
     }
 
     fn zoom_best_fit(&self) {
@@ -420,31 +415,39 @@ impl LpWindow {
     }
 
     async fn pick_file(&self) {
-        let filter_store = gio::ListStore::new(gtk::FileFilter::static_type());
+        let filter_list = gio::ListStore::new(gtk::FileFilter::static_type());
 
-        let filter = gtk::FileFilter::new();
-        filter.set_property("name", &String::from("Supported image files"));
-        filter.add_mime_type("image/*");
-        filter_store.append(&filter);
+        let filter_supported_formats = gtk::FileFilter::new();
+        filter_supported_formats.set_name(Some(&gettext("Supported image formats")));
+        filter_supported_formats.add_mime_type("image/*");
+        // Manual support for Farbfeld since it has no mime type
+        filter_supported_formats.add_suffix("ff");
+
+        let filter_all_files = gtk::FileFilter::new();
+        filter_all_files.set_name(Some(&gettext("All files")));
+        filter_all_files.add_pattern("*");
+
+        filter_list.append(&filter_supported_formats);
+        filter_list.append(&filter_all_files);
 
         let chooser = gtk::FileDialog::builder()
-            .title(i18n("Open Image"))
-            .filters(&filter_store)
+            .title(gettext("Open Image"))
+            .filters(&filter_list)
+            .default_filter(&filter_supported_formats)
             .modal(true)
             .build();
 
         if let Ok(file) = chooser.open_future(Some(self)).await {
-            self.set_image_from_path(&file.path().unwrap());
+            self.image_view().set_image_from_file(file);
         } else {
-            // TODO: ui error
-            log::error!("Failed to choose file");
+            log::debug!("File dialog canceled or file not readable");
         }
     }
 
     async fn open_with(&self) {
         let imp = self.imp();
 
-        if let Some(ref file) = imp.image_view.current_path().map(gio::File::for_path) {
+        if let Some(ref file) = imp.image_view.current_file() {
             let launcher = gtk::FileLauncher::new(Some(file));
             if let Err(e) = launcher.launch_future(Some(self)).await {
                 if !e.matches(gtk::DialogError::Dismissed) {
@@ -482,13 +485,16 @@ impl LpWindow {
         if let Err(e) = imp.image_view.copy() {
             log::error!("Failed to copy to clipboard: {}", e);
         } else {
-            self.show_toast(i18n("Image copied to clipboard"), adw::ToastPriority::High);
+            self.show_toast(
+                gettext("Image copied to clipboard"),
+                adw::ToastPriority::High,
+            );
         }
     }
 
     async fn trash(&self) {
         let image_view = self.image_view();
-        let (Some(file), Some(path)) = (image_view.current_file(), image_view.current_path())
+        let (Some(file), Some(path)) = (image_view.current_file(), image_view.current_file().and_then(|x| x.path()))
             else { log::error!("No file to trash"); return; };
 
         let result = file.trash_future(glib::Priority::default()).await;
@@ -496,19 +502,19 @@ impl LpWindow {
         match result {
             Ok(()) => {
                 let toast = adw::Toast::builder()
-                    .title(i18n("Image moved to trash"))
-                    .button_label(i18n("Undo"))
+                    .title(gettext("Image moved to trash"))
+                    .button_label(gettext("Undo"))
                     .build();
                 toast.connect_button_clicked(glib::clone!(@weak self as win => move |_| {
                     let path = path.clone();
                     spawn(async move {
                         let result = crate::util::untrash(&path).await;
                         match result {
-                            Ok(()) => win.image_view().set_image_from_path(&path),
+                            Ok(()) => win.image_view().set_image_from_file(gio::File::for_path(&path)),
                             Err(err) => {
                                 log::error!("Failed to untrash {path:?}: {err}");
                                 win.show_toast(
-                                    i18n("Failed to restore image from trash"),
+                                    gettext("Failed to restore image from trash"),
                                     adw::ToastPriority::High,
                                 );
                             }
@@ -523,7 +529,7 @@ impl LpWindow {
                 } else {
                     log::error!("Failed to delete file {path:?}: {err}");
                     self.show_toast(
-                        i18n("Failed to move image to trash"),
+                        gettext("Failed to move image to trash"),
                         adw::ToastPriority::Normal,
                     );
                 }
@@ -538,8 +544,8 @@ impl LpWindow {
         let dialog = adw::MessageDialog::builder()
             .modal(true)
             .transient_for(self)
-            .heading(i18n("Permanently Delete Image?"))
-            .body(i18n_f(
+            .heading(gettext("Permanently Delete Image?"))
+            .body(gettext_f(
                 "The image “{}” can only be deleted permanently.",
                 &[&PathBuf::from(&path.file_name().unwrap_or_default())
                     .display()
@@ -547,7 +553,10 @@ impl LpWindow {
             ))
             .build();
 
-        dialog.add_responses(&[("cancel", &i18n("Cancel")), ("delete", &i18n("Delete"))]);
+        dialog.add_responses(&[
+            ("cancel", &gettext("Cancel")),
+            ("delete", &gettext("Delete")),
+        ]);
         dialog.set_response_appearance("delete", adw::ResponseAppearance::Destructive);
 
         if "delete" == dialog.choose_future().await {
@@ -556,12 +565,15 @@ impl LpWindow {
 
             if let Err(err) = result {
                 log::error!("Failed to delete file {path:?}: {err}");
-                self.show_toast(i18n("Failed to delete image"), adw::ToastPriority::Normal);
+                self.show_toast(
+                    gettext("Failed to delete image"),
+                    adw::ToastPriority::Normal,
+                );
             }
         }
     }
 
-    fn image_view(&self) -> LpImageView {
+    pub fn image_view(&self) -> LpImageView {
         self.imp().image_view.clone()
     }
 
@@ -574,14 +586,6 @@ impl LpWindow {
         imp.toast_overlay.add_toast(toast);
     }
 
-    pub fn set_image_from_path(&self, path: &Path) {
-        let imp = self.imp();
-
-        log::debug!("Loading file: {path:?}");
-        imp.image_view.set_image_from_path(path);
-        self.set_actions_enabled(true);
-    }
-
     pub fn set_actions_enabled(&self, enabled: bool) {
         self.action_set_enabled("win.open-with", enabled);
         self.action_set_enabled("win.set-background", enabled);
@@ -591,7 +595,7 @@ impl LpWindow {
         self.action_set_enabled("win.copy", enabled);
         self.action_set_enabled("win.trash", enabled);
         self.action_set_enabled("win.zoom-best-fit", enabled);
-        self.action_set_enabled("win.zoom-to", enabled);
+        self.action_set_enabled("win.zoom-to-exact", enabled);
         self.action_set_enabled("win.toggle-properties", enabled);
     }
 
@@ -599,10 +603,17 @@ impl LpWindow {
     fn images_available(&self) {
         let imp = self.imp();
 
-        if imp.image_view.current_page().is_some() {
+        let shows_image = imp.image_view.current_page().is_some();
+
+        imp.properties_button.set_sensitive(shows_image);
+        self.set_actions_enabled(shows_image);
+
+        if shows_image {
             imp.stack.set_visible_child(&*imp.image_view);
             imp.image_view.grab_focus();
+            self.queue_hide_controls();
         } else {
+            imp.properties_button.set_active(false);
             imp.stack.set_visible_child(&*imp.status_page);
             imp.status_page.grab_focus();
         }
@@ -661,6 +672,103 @@ impl LpWindow {
         self.action_set_enabled("win.zoom-in", can_zoom_in);
     }
 
+    fn set_control_opacity(&self, opacity: f64) {
+        self.image_view().controls_box_start().set_opacity(opacity);
+        self.image_view().controls_box_end().set_opacity(opacity);
+    }
+
+    fn controls_opacity(&self) -> f64 {
+        self.image_view().controls_box_start().opacity()
+    }
+
+    /// Animation to show controls
+    fn show_controls_animation(&self) -> &adw::TimedAnimation {
+        self.imp().show_controls_animation.get_or_init(|| {
+            let target = adw::CallbackAnimationTarget::new(glib::clone!(
+                @weak self as obj => move |opacity| obj.set_control_opacity(opacity)
+            ));
+
+            adw::TimedAnimation::builder()
+                .duration(SHOW_CONTROLS_ANIMATION_DURATION)
+                .widget(&self.image_view())
+                .target(&target)
+                .value_to(1.)
+                .build()
+        })
+    }
+
+    /// Animation to hide controls
+    fn hide_controls_animation(&self) -> &adw::TimedAnimation {
+        self.imp().hide_controls_animation.get_or_init(|| {
+            let target = adw::CallbackAnimationTarget::new(glib::clone!(
+                @weak self as obj => move |opacity| obj.set_control_opacity(opacity)
+            ));
+
+            adw::TimedAnimation::builder()
+                .duration(HIDE_CONTROLS_ANIMATION_DURATION)
+                .widget(&self.image_view())
+                .target(&target)
+                .value_to(0.)
+                .build()
+        })
+    }
+
+    /// Queue a fade animation to play after `HIDE_CONTROLS_IDLE_TIMEOUT` seconds of inactivity
+    fn queue_hide_controls(&self) {
+        self.dequeue_hide_controls();
+
+        let new_timeout = glib::timeout_add_seconds_local_once(
+            HIDE_CONTROLS_IDLE_TIMEOUT,
+            glib::clone!(@weak self as win => move || {
+                win.imp().hide_controls_timeout.take();
+                win.hide_controls();
+            }),
+        );
+
+        self.imp().hide_controls_timeout.replace(Some(new_timeout));
+    }
+
+    fn dequeue_hide_controls(&self) {
+        if let Some(current_timeout) = self.imp().hide_controls_timeout.take() {
+            current_timeout.remove();
+        }
+    }
+
+    fn is_controls_visible(&self) -> bool {
+        if self.hide_controls_animation().state() == adw::AnimationState::Playing {
+            return false;
+        }
+
+        self.image_view().controls_box_start().opacity() == 1.
+            || self.show_controls_animation().state() == adw::AnimationState::Playing
+    }
+
+    /// Start animation to show controls
+    fn show_controls(&self) {
+        if !self.is_controls_visible() {
+            self.hide_controls_animation().pause();
+            self.show_controls_animation()
+                .set_value_from(self.controls_opacity());
+            self.show_controls_animation().play();
+        }
+    }
+
+    /// Start animation to hide controls
+    fn hide_controls(&self) {
+        if self.is_controls_visible() {
+            self.show_controls_animation().pause();
+            self.hide_controls_animation()
+                .set_value_from(self.controls_opacity());
+            self.hide_controls_animation().play();
+        }
+    }
+
+    /// Whether or not the window is showing images
+    fn is_showing_image(&self) -> bool {
+        let imp = self.imp();
+        imp.stack.visible_child().as_ref() == Some(imp.image_view.upcast_ref())
+    }
+
     // In the LpWindow UI file we define a `gtk::Expression`s
     // that is a closure. This closure takes the current `gio::File`
     // and processes it to return a window title.
@@ -670,28 +778,41 @@ impl LpWindow {
     // have multiple `match` or `if let` branches, and without needing
     // to unwrap.
     #[template_callback]
-    fn window_title(&self, path: glib::variant::Variant) -> String {
+    fn window_title(&self, file: Option<&gio::File>) -> String {
         // ensure that templates are initialized
-        if path.as_maybe().is_none() {
-            i18n("Loupe")
+        if file.is_none() {
+            gettext("Loupe")
         } else {
             self.imp()
                 .image_view
                 .current_file()
                 .and_then(|f| util::get_file_display_name(&f)) // If the file exists, get display name
-                .unwrap_or_else(|| i18n("Loupe")) // Return that or the default if there's nothing
+                .unwrap_or_else(|| gettext("Loupe")) // Return that or the default if there's nothing
         }
     }
 
-    // We also have a closure that returns `adw::FlapFoldPolicy`.
-    // if we aren't fullscreened, or if the properties are revealed,
-    // we unfold the main flap. Otherwise we're always folded.
     #[template_callback]
-    fn fold_policy(&self, fullscreened: bool, properties_revealed: bool) -> adw::FlapFoldPolicy {
-        if !fullscreened || properties_revealed {
-            adw::FlapFoldPolicy::Never
+    fn on_motion_cb(&self) {
+        let controller = &self.imp().motion_controller;
+        if controller.current_event_time() == 0 {
+            return;
+        }
+
+        self.show_controls();
+        // Only queue animation if not over controls and there is an image shown
+        if !self
+            .image_view()
+            .controls_box_start_events()
+            .contains_pointer()
+            && !self
+                .image_view()
+                .controls_box_end_events()
+                .contains_pointer()
+            && self.is_showing_image()
+        {
+            self.queue_hide_controls();
         } else {
-            adw::FlapFoldPolicy::Always
+            self.dequeue_hide_controls();
         }
     }
 }
